@@ -23,6 +23,7 @@
 
 #include "kcodecs.h"
 #include "kcharsets.h"
+#include "kcharsets_p.h"
 #include "kcodecs_debug.h"
 #include "kcodecs_p.h"
 #include "kcodecsbase64.h"
@@ -40,7 +41,8 @@
 
 #include <QDebug>
 #include <QHash>
-#include <QTextCodec>
+#include <QStringDecoder>
+#include <QStringEncoder>
 
 #if defined(Q_OS_WIN)
 #define strncasecmp _strnicmp
@@ -52,16 +54,41 @@ static QList<QByteArray> charsetCache;
 
 QByteArray cachedCharset(const QByteArray &name)
 {
-    for (const QByteArray &charset : qAsConst(charsetCache)) {
-        if (qstricmp(name.data(), charset.data()) == 0) {
-            return charset;
-        }
+    auto it = std::find_if(charsetCache.cbegin(), charsetCache.cend(), [&name](const QByteArray &charset) {
+        return qstricmp(name.data(), charset.data()) == 0;
+    });
+    if (it != charsetCache.cend()) {
+        return *it;
     }
 
     charsetCache.append(name.toUpper());
     return charsetCache.last();
 }
 
+namespace CodecNames
+{
+QByteArray utf8()
+{
+    return QByteArrayLiteral("UTF-8");
+}
+}
+
+Q_REQUIRED_RESULT
+QByteArray updateEncodingCharset(const QByteArray &currentCharset, const QByteArray &nextCharset)
+{
+    if (!nextCharset.isEmpty()) {
+        if (currentCharset.isEmpty()) {
+            return nextCharset;
+        }
+        if (currentCharset != nextCharset) {
+            // only one charset per string supported, so change to superset charset UTF-8,
+            // which should  cover any possible chars
+            return CodecNames::utf8();
+        }
+    }
+    return currentCharset;
+}
+
 } // namespace KCodecs
 
 /******************************** KCodecs ********************************/
@@ -94,14 +121,6 @@ QByteArray KCodecs::base64Encode(const QByteArray &in)
     return codec->encode(in);
 }
 
-#if KCODECS_BUILD_DEPRECATED_SINCE(5, 5)
-QByteArray KCodecs::base64Encode(const QByteArray &in, bool insertLFs)
-{
-    Q_UNUSED(insertLFs);
-    return base64Encode(in);
-}
-#endif
-
 void KCodecs::base64Encode(const QByteArray &in, QByteArray &out, bool insertLFs)
 {
     Q_UNUSED(insertLFs);
@@ -119,21 +138,6 @@ void KCodecs::base64Decode(const QByteArray &in, QByteArray &out)
     out = base64Decode(in);
 }
 
-#if KCODECS_BUILD_DEPRECATED_SINCE(5, 56)
-QByteArray KCodecs::uuencode(const QByteArray &in)
-{
-    Codec *codec = Codec::codecForName("x-uuencode");
-    return codec->encode(in);
-}
-#endif
-
-#if KCODECS_BUILD_DEPRECATED_SINCE(5, 56)
-void KCodecs::uuencode(const QByteArray &in, QByteArray &out)
-{
-    out = uuencode(in);
-}
-#endif
-
 QByteArray KCodecs::uudecode(const QByteArray &in)
 {
     Codec *codec = Codec::codecForName("x-uuencode");
@@ -177,7 +181,7 @@ bool parseEncodedWord(const char *&scursor,
         return false;
     }
 
-    // remember start of charset (ie. just after the initial "=?") and
+    // remember start of charset (i.e. just after the initial "=?") and
     // language (just after the first '*') fields:
     const char *charsetStart = scursor;
     const char *languageStart = nullptr;
@@ -288,26 +292,25 @@ bool parseEncodedWord(const char *&scursor,
     assert(dec);
 
     // try if there's a (text)codec for the charset found:
-    bool matchOK = false;
     QByteArray cs;
-    QTextCodec *textCodec = nullptr;
+    QStringDecoder textCodec;
     if (charsetOption == KCodecs::ForceDefaultCharset || maybeCharset.isEmpty()) {
-        textCodec = KCharsets::charsets()->codecForName(QLatin1String(defaultCS), matchOK);
+        textCodec = QStringDecoder(defaultCS.constData());
         cs = cachedCharset(defaultCS);
     } else {
-        textCodec = KCharsets::charsets()->codecForName(QLatin1String(maybeCharset), matchOK);
-        if (!matchOK) { // no suitable codec found => use default charset
-            textCodec = KCharsets::charsets()->codecForName(QLatin1String(defaultCS), matchOK);
+        textCodec = QStringDecoder(maybeCharset.constData());
+        if (!textCodec.isValid()) { // no suitable codec found => use default charset
+            textCodec = QStringDecoder(defaultCS.constData());
             cs = cachedCharset(defaultCS);
         } else {
             cs = cachedCharset(maybeCharset);
         }
     }
     if (usedCS) {
-        *usedCS = cs;
+        *usedCS = updateEncodingCharset(*usedCS, cs);
     }
 
-    if (!matchOK || !textCodec) {
+    if (!textCodec.isValid()) {
         // qCDebug(KCODECS_LOG) << "Unknown charset" << maybeCharset;
         delete dec;
         return false;
@@ -331,7 +334,7 @@ bool parseEncodedWord(const char *&scursor,
         qWarning() << codec->name() << "codec lies about its maxDecodedSizeFor(" << encodedTextLength << ")\nresult may be truncated";
     }
 
-    *result = textCodec->toUnicode(buffer.data(), bbegin - buffer.data());
+    *result = textCodec.decode(QByteArrayView(buffer.data(), bbegin - buffer.data()));
 
     // qCDebug(KCODECS_LOG) << "result now: \"" << result << "\"";
     // cleanup:
@@ -348,7 +351,7 @@ bool parseEncodedWord(const char *&scursor,
 QString KCodecs::decodeRFC2047String(const QString &msg)
 {
     QByteArray usedCS;
-    return decodeRFC2047String(msg.toUtf8(), &usedCS, "utf-8", NoOption);
+    return decodeRFC2047String(msg.toUtf8(), &usedCS, CodecNames::utf8(), NoOption);
 }
 
 QString KCodecs::decodeRFC2047String(const QByteArray &src, QByteArray *usedCS, const QByteArray &defaultCS, CharsetOption charsetOption)
@@ -358,6 +361,9 @@ QString KCodecs::decodeRFC2047String(const QByteArray &src, QByteArray *usedCS,
     const char *scursor = src.constData();
     const char *send = scursor + src.length();
     bool onlySpacesSinceLastWord = false;
+    if (usedCS) {
+        usedCS->clear();
+    }
 
     while (scursor != send) {
         // space
@@ -398,9 +404,12 @@ QString KCodecs::decodeRFC2047String(const QByteArray &src, QByteArray *usedCS,
     // If there are any chars that couldn't be decoded in UTF-8,
     // fallback to local codec
     const QString tryUtf8 = QString::fromUtf8(result);
-    if (tryUtf8.contains(0xFFFD)) {
-        QTextCodec *codec = QTextCodec::codecForLocale();
-        return codec->toUnicode(result);
+    if (tryUtf8.contains(QChar(0xFFFD))) {
+        QStringDecoder codec(QStringDecoder::System);
+        if (usedCS) {
+            *usedCS = updateEncodingCharset(*usedCS, cachedCharset(codec.name()));
+        }
+        return codec.decode(result);
     } else {
         return tryUtf8;
     }
@@ -412,32 +421,29 @@ QByteArray KCodecs::encodeRFC2047String(const QString &src, const QByteArray &ch
     int start = 0;
     int end = 0;
     bool nonAscii = false;
-    bool ok = true;
     bool useQEncoding = false;
 
-    // fromLatin1() is safe here, codecForName() uses toLatin1() internally
-    const QTextCodec *codec = KCharsets::charsets()->codecForName(QString::fromLatin1(charset), ok);
+    QStringEncoder codec(charset.constData());
 
     QByteArray usedCS;
-    if (!ok) {
+    if (!codec.isValid()) {
         // no codec available => try local8Bit and hope the best ;-)
-        codec = QTextCodec::codecForLocale();
-        usedCS = codec->name();
+        codec = QStringEncoder(QStringEncoder::System);
+        usedCS = codec.name();
     } else {
-        Q_ASSERT(codec);
+        Q_ASSERT(codec.isValid());
         if (charset.isEmpty()) {
-            usedCS = codec->name();
+            usedCS = codec.name();
         } else {
             usedCS = charset;
         }
     }
 
-    QTextCodec::ConverterState converterState(QTextCodec::IgnoreHeader);
-    QByteArray encoded8Bit = codec->fromUnicode(src.constData(), src.length(), &converterState);
-    if (converterState.invalidChars > 0) {
-        usedCS = "utf-8";
-        codec = QTextCodec::codecForName(usedCS);
-        encoded8Bit = codec->fromUnicode(src);
+    QByteArray encoded8Bit = codec.encode(src);
+    if (codec.hasError()) {
+        usedCS = CodecNames::utf8();
+        codec = QStringEncoder(QStringEncoder::Utf8);
+        encoded8Bit = codec.encode(src);
     }
 
     if (usedCS.contains("8859-")) { // use "B"-Encoding for non iso-8859-x charsets
@@ -480,10 +486,9 @@ QByteArray KCodecs::encodeRFC2047String(const QString &src, const QByteArray &ch
         if (useQEncoding) {
             result += "?Q?";
 
-            char c;
             char hexcode; // "Q"-encoding implementation described in RFC 2047
             for (int i = start; i < end; i++) {
-                c = encoded8Bit[i];
+                const char c = encoded8Bit[i];
                 if (c == ' ') { // make the result readable with not MIME-capable readers
                     result += '_';
                 } else {
@@ -544,9 +549,7 @@ static void createCodecs()
 
 static void cleanupCodecs()
 {
-    for (auto iter = allCodecs->begin(); iter != allCodecs->end(); ++iter) {
-        delete (*iter);
-    }
+    qDeleteAll(*allCodecs);
     delete allCodecs;
     allCodecs = nullptr;
 }
@@ -562,7 +565,7 @@ KCodecs::Codec *KCodecs::Codec::codecForName(const char *name)
 
 KCodecs::Codec *KCodecs::Codec::codecForName(const QByteArray &name)
 {
-    QMutexLocker locker(dictLock); // protect "allCodecs"
+    QMutexLocker locker(dictLock()); // protect "allCodecs"
     if (!allCodecs) {
         allCodecs = new QHash<QByteArray, Codec *>();
         qAddPostRoutine(cleanupCodecs);
@@ -581,8 +584,8 @@ KCodecs::Codec *KCodecs::Codec::codecForName(const QByteArray &name)
 bool KCodecs::Codec::encode(const char *&scursor, const char *const send, char *&dcursor, const char *const dend, NewlineType newline) const
 {
     // get an encoder:
-    QScopedPointer<Encoder> enc(makeEncoder(newline));
-    if (enc.isNull()) {
+    std::unique_ptr<Encoder> enc(makeEncoder(newline));
+    if (!enc) {
         qWarning() << "makeEncoder failed for" << name();
         return false;
     }
@@ -653,8 +656,8 @@ QByteArray KCodecs::Codec::decode(const QByteArray &src, NewlineType newline) co
 bool KCodecs::Codec::decode(const char *&scursor, const char *const send, char *&dcursor, const char *const dend, NewlineType newline) const
 {
     // get a decoder:
-    QScopedPointer<Decoder> dec(makeDecoder(newline));
-    assert(!dec.isNull());
+    std::unique_ptr<Decoder> dec(makeDecoder(newline));
+    assert(dec);
 
     // decode and check for output buffer overflow:
     while (!dec->decode(scursor, send, dcursor, dend)) {
@@ -718,7 +721,7 @@ bool KCodecs::Encoder::flushOutputBuffer(char *&dcursor, const char *const dend)
 
     // calculate the number of missing chars:
     int numCharsLeft = d->outputBufferCursor - i;
-    // push the remaining chars to the begin of the buffer:
+    // push the remaining chars to the beginning of the buffer:
     if (numCharsLeft) {
         ::memmove(d->outputBuffer, d->outputBuffer + i, numCharsLeft);
     }
