@@ -1,34 +1,25 @@
 /*
-   Copyright (C) 2000-2001 Dawit Alemayehu <adawit@kde.org>
-   Copyright (C) 2001 Rik Hemsley (rikkus) <rik@kde.org>
-   Copyright (c) 2001-2002 Marc Mutz <mutz@kde.org>
+    SPDX-FileCopyrightText: 2000-2001 Dawit Alemayehu <adawit@kde.org>
+    SPDX-FileCopyrightText: 2001 Rik Hemsley (rikkus) <rik@kde.org>
+    SPDX-FileCopyrightText: 2001-2002 Marc Mutz <mutz@kde.org>
 
-   This program is free software; you can redistribute it and/or modify
-   it under the terms of the GNU Lesser General Public License (LGPL)
-   version 2 as published by the Free Software Foundation.
+    SPDX-License-Identifier: LGPL-2.0-only
 
-   This program is distributed in the hope that it will be useful,
-   but WITHOUT ANY WARRANTY; without even the implied warranty of
-   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-   GNU Lesser General Public License for more details.
-
-   You should have received a copy of the GNU Lesser General Public
-   License along with this program; if not, write to the Free Software
-   Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
-
-   The quoted-printable codec as described in RFC 2045, section 6.7. is by
-   Rik Hemsley (C) 2001.
+    The quoted-printable codec as described in RFC 2045, section 6.7. is by
+    Rik Hemsley (C) 2001.
 */
 
 #ifndef KCODECS_H
 #define KCODECS_H
 
-#define KBase64 KCodecs
-
 #include <kcodecs_export.h>
+
 #include <QString>
 
-template<typename T, typename U> class QHash;
+#include <memory>
+
+template<typename T, typename U>
+class QHash;
 
 class QByteArray;
 class QIODevice;
@@ -65,7 +56,6 @@ class QIODevice;
  */
 namespace KCodecs
 {
-
 /**
  * Encodes the given data using the quoted-printable algorithm.
  *
@@ -75,8 +65,7 @@ namespace KCodecs
  *                breaks, too.
  * @return        quoted-printable encoded string.
  */
-KCODECS_EXPORT QByteArray quotedPrintableEncode(const QByteArray &in,
-        bool useCRLF = true);
+KCODECS_EXPORT QByteArray quotedPrintableEncode(const QByteArray &in, bool useCRLF = true);
 
 /**
  * Encodes the given data using the quoted-printable algorithm.
@@ -96,8 +85,7 @@ KCODECS_EXPORT QByteArray quotedPrintableEncode(const QByteArray &in,
  *                CRLF line breaks and the output will have CRLF line
  *                breaks, too.
  */
-KCODECS_EXPORT void quotedPrintableEncode(const QByteArray &in, QByteArray &out,
-        bool useCRLF);
+KCODECS_EXPORT void quotedPrintableEncode(const QByteArray &in, QByteArray &out, bool useCRLF);
 
 /**
  * Decodes a quoted-printable encoded data.
@@ -129,36 +117,6 @@ KCODECS_EXPORT QByteArray quotedPrintableDecode(const QByteArray &in);
  */
 KCODECS_EXPORT void quotedPrintableDecode(const QByteArray &in, QByteArray &out);
 
-/**
- * Encodes the given data using the uuencode algorithm.
- *
- * The output is split into lines starting with the number of
- * encoded octets in the line and ending with a newline.  No
- * line is longer than 45 octets (60 characters), excluding the
- * line terminator.
- *
- * @param in   data to be uuencoded
- * @return     uuencoded string.
- */
-KCODECS_EXPORT QByteArray uuencode(const QByteArray &in);
-
-/**
- * Encodes the given data using the uuencode algorithm.
- *
- * Use this function if you want the result of the encoding
- * to be placed in another array and cut down the number of
- * copy operation that have to be performed in the process.
- * This is the preffered method for encoding binary data.
- *
- * NOTE: the output array is first reset and then resized
- * appropriately before use, hence, all data stored in the
- * output array will be lost.
- *
- * @param in   data to be uuencoded.
- * @param out  uudecoded data.
- */
-KCODECS_EXPORT void uuencode(const QByteArray &in, QByteArray &out);
-
 /**
  * Decodes the given data using the uudecode algorithm.
  *
@@ -208,13 +166,6 @@ KCODECS_EXPORT void uudecode(const QByteArray &in, QByteArray &out);
  */
 KCODECS_EXPORT QByteArray base64Encode(const QByteArray &in);
 
-/**
- * @copydoc
- * KCodecs::base64Encode(QByteArray)
- * @deprecated Use KCodecs::base64Encode(QByteArray) instead.
- */
-KCODECS_DEPRECATED_EXPORT QByteArray base64Encode(const QByteArray &in, bool insertLFs);
-
 /**
  * Encodes the given data using the base64 algorithm.
  *
@@ -236,8 +187,7 @@ KCODECS_DEPRECATED_EXPORT QByteArray base64Encode(const QByteArray &in, bool ins
  * @param out       encoded data.
  * @param insertLFs limit the number of characters per line.
  */
-KCODECS_EXPORT void base64Encode(const QByteArray &in, QByteArray &out,
-                                 bool insertLFs = false);
+KCODECS_EXPORT void base64Encode(const QByteArray &in, QByteArray &out, bool insertLFs = false);
 
 /**
  * Decodes the given data that was encoded using the
@@ -280,10 +230,9 @@ KCODECS_EXPORT QString decodeRFC2047String(const QString &text);
  * Charset options for RFC2047 encoder
  * @since 5.5
  */
-enum CharsetOption
-{
-    NoOption = 0,              /// No special option
-    ForceDefaultCharset = 1    /// Force use of the default charset
+enum CharsetOption {
+    NoOption = 0, /// No special option
+    ForceDefaultCharset = 1, /// Force use of the default charset
 };
 
 /**
@@ -291,7 +240,9 @@ enum CharsetOption
  *  =?charset?[qb]?encoded?=
  *
  * @param src       source string.
- * @param usedCS    the detected charset is returned here
+ * @param usedCS    the name of any detected charset or, in case of multiple
+ *                  different ones, "UTF-8" as that of a super charset is
+ *                  returned here
  * @param defaultCS the charset to use in case the detected
  *                  one isn't known to us.
  * @param option    options for the encoder
@@ -299,7 +250,8 @@ enum CharsetOption
  * @return the decoded string.
  * @since 5.5
  */
-KCODECS_EXPORT QString decodeRFC2047String(const QByteArray &src, QByteArray *usedCS,
+KCODECS_EXPORT QString decodeRFC2047String(const QByteArray &src,
+                                           QByteArray *usedCS,
                                            const QByteArray &defaultCS = QByteArray(),
                                            CharsetOption option = NoOption);
 
@@ -319,6 +271,16 @@ KCODECS_EXPORT QString decodeRFC2047String(const QByteArray &src, QByteArray *us
  */
 KCODECS_EXPORT QByteArray encodeRFC2047String(const QString &src, const QByteArray &charset);
 
+/**
+ * Decodes the given data that was encoded using the
+ * base45 codec.
+ *
+ * @param in   data to be decoded.
+ * @return     decoded string.
+ * @since 5.84
+ * @see https://datatracker.ietf.org/doc/draft-faltstrom-base45/
+ */
+KCODECS_EXPORT QByteArray base45Decode(const QByteArray &in);
 
 class Encoder;
 class EncoderPrivate;
@@ -369,7 +331,7 @@ class KCODECS_EXPORT Codec
 public:
     enum NewlineType {
         NewlineLF,
-        NewlineCRLF
+        NewlineCRLF,
     };
 
     /**
@@ -460,9 +422,7 @@ public:
       @return false if the encoded data didn't fit into the output buffer;
       true otherwise.
     */
-    virtual bool encode(const char *&scursor, const char *const send,
-                        char *&dcursor, const char *const dend,
-                        NewlineType newline = NewlineLF) const;
+    virtual bool encode(const char *&scursor, const char *const send, char *&dcursor, const char *const dend, NewlineType newline = NewlineLF) const;
 
     /**
       Convenience wrapper that can be used for small chunks of data
@@ -500,9 +460,7 @@ public:
       @return false if the decoded data didn't fit into the output buffer;
       true otherwise.
     */
-    virtual bool decode(const char *&scursor, const char *const send,
-                        char *&dcursor, const char *const dend,
-                        NewlineType newline = NewlineLF) const;
+    virtual bool decode(const char *&scursor, const char *const send, char *&dcursor, const char *const dend, NewlineType newline = NewlineLF) const;
 
     /**
       Even more convenient, but also a bit slower and more memory
@@ -514,8 +472,7 @@ public:
       @param src is a QByteArray containing the data to encode.
       @param newline whether make new lines using @ref CRLF, or @ref LF (default is @ref LF).
     */
-    virtual QByteArray encode(const QByteArray &src,
-                              NewlineType newline = NewlineLF) const;
+    virtual QByteArray encode(const QByteArray &src, NewlineType newline = NewlineLF) const;
 
     /**
       Even more convenient, but also a bit slower and more memory
@@ -527,8 +484,7 @@ public:
       @param src is a QByteArray containing the data to decode.
       @param newline whether make new lines using @ref CRLF, or @ref LF (default is @ref LF).
     */
-    virtual QByteArray decode(const QByteArray &src,
-                              NewlineType newline = NewlineLF) const;
+    virtual QByteArray decode(const QByteArray &src, NewlineType newline = NewlineLF) const;
 
     /**
       Returns the name of the encoding. Guaranteed to be lowercase.
@@ -538,13 +494,17 @@ public:
     /**
       Destroys the codec.
     */
-    virtual ~Codec() {}
+    virtual ~Codec()
+    {
+    }
 
 protected:
     /**
       Constructs the codec.
     */
-    Codec() {}
+    Codec()
+    {
+    }
 };
 
 /**
@@ -569,7 +529,7 @@ protected:
   often as necessary, then call finalize (most often a single
   call suffices, but it might be that during that call the output
   buffer is filled, so you should be prepared to call finalize
-  as often as necessary, ie. until it returns @p true).
+  as often as necessary, i.e. until it returns @p true).
 
   @section Return Values
 
@@ -658,8 +618,7 @@ public:
       @param dcursor is a pointer to the start of the output buffer.
       @param dend is a pointer to the end of the output buffer.
     */
-    virtual bool decode(const char *&scursor, const char *const send,
-                        char *&dcursor, const char *const dend) = 0;
+    virtual bool decode(const char *&scursor, const char *const send, char *&dcursor, const char *const dend) = 0;
 
     /**
       Call this method to finalize the output stream. Writes all
@@ -673,7 +632,7 @@ public:
 
 protected:
     //@cond PRIVATE
-    DecoderPrivate * const d;
+    std::unique_ptr<DecoderPrivate> const d;
     //@endcond
 };
 
@@ -688,7 +647,6 @@ protected:
 */
 class KCODECS_EXPORT Encoder
 {
-
 protected:
     friend class Codec;
     friend class EncoderPrivate;
@@ -715,8 +673,7 @@ public:
       @param dcursor is a pointer to the start of the output buffer.
       @param dend is a pointer to the end of the output buffer.
     */
-    virtual bool encode(const char *&scursor, const char *const send,
-                        char *&dcursor, const char *const dend) = 0;
+    virtual bool encode(const char *&scursor, const char *const send, char *&dcursor, const char *const dend) = 0;
 
     /**
       Call this method to finalize the output stream. Writes all remaining
@@ -732,7 +689,7 @@ protected:
       The maximum number of characters permitted in the output buffer.
     */
     enum {
-        maxBufferedChars = 8  /**< Eight */
+        maxBufferedChars = 8, /**< Eight */
     };
 
     /**
@@ -770,7 +727,7 @@ protected:
 
 protected:
     //@cond PRIVATE
-    EncoderPrivate * const d;
+    std::unique_ptr<EncoderPrivate> const d;
     //@endcond
 };
 
